var test_environment = false;
// Globally available variables, some not (yet) initialized !
var param_map, param_map_MF_temp = null, param_map_nMF_temp = null;
var currentDeviceSVG;
const WITH_MOSFET = 1;
const NO_MOSFET = 2;
var toolMOSFETVersion = WITH_MOSFET; // Tool starts with MOSFET version. Can be changed using GUI.
var useRecommendedResistorValues = true; // Use either calculated/recommended values for R1 and R2 or manually entered ones.
var recommendedNonMF = true, recommendedMF = true, firstRunNonMF = true, firstRunMF = true;
var NDEC = 2, powerData;
var canvas1, canvas2, chart1, chart2;
var vsmin, vsmax, vout, iout, tambient, rthja, pvmax, v_overhead;
var rdson, vgstyp;
var r1, r2, r1_selected, r2_selected, pr1_max, pr1_max_index, pr2_max, pr2_max_index, p1114max, p1114max_index, rset, cfg_setting, output_cfg;
var vin_set, k1, k2, ipsmax, vgs_pwr_sh, rshunt, vsmax_spec, vsmin_spec, iout_min, iout_max, vps_cc, vout_cc, tjmax;
var result, result_description;
const mid_current = 0.18; // Current value which is used to decide whether to use OUTH or OUTL.

var last_updated_param_id = '/';

/**
 * This function fetches the data from input fields of an HTML page.
 * The variables are in the same order as inside the HTML page.
 * Not every variable's value must be fetched, some are calculated.
 * Some variables' values on the HTML page are expressed in non-base unit prefix,
 * so they are going to be converted to the base unit prefix here and then saved.
 */
function fetchData()
{
    guardDependentParameters('vsmin', 'vsmax', last_updated_param_id);
    guardDependentParametersLowerFor('vout', 'vsmax', 'vps_cc', last_updated_param_id);

    // Input parameters:
    vsmin = getParamValue('vsmin');
    vsmax = getParamValue('vsmax');
    vout = getParamValue('vout');
    iout = fromMili( getParamValue('iout') );
    tambient = getParamValue('tambient');
    rthja = getParamValue('rthja');
    v_overhead = getParamValue('v_overhead');
    rdson = getParamValue('rdson');
    vgstyp = getParamValue('vgstyp');
    r1_selected = getParamValue('r1_selected');
    r2_selected = getParamValue('r2_selected');

    // Product parameters (TLD1114-EP):
    vin_set = getParamValue('vin_set');
    k1 = getParamValue('k1');
    k2 = getParamValue('k2');
    ipsmax = fromMili( getParamValue('ipsmax') );
    vgs_pwr_sh = getParamValue('vgs_pwr_sh');
    rshunt = getParamValue('rshunt');
    vsmax_spec = getParamValue('vsmax_spec');
    vsmin_spec = getParamValue('vsmin_spec');
    iout_min = fromMili( getParamValue('iout_min') );
    // iout_max = fromMili( getParamValue('iout_max') ); // value not used anywhere
    vps_cc = getParamValue('vps_cc');
    vout_cc = getParamValue('vout_cc');
    tjmax = getParamValue('tjmax');

    /* Tool version without MOSFET -> no rdson, no vgstyp, no v_overhead, no r2. */
    if (toolMOSFETVersion == NO_MOSFET) {
        rdson = 0.0;
        vgstyp = 0.0;
        v_overhead = 0.0;
        r2 = 0.000000001;
        r2_selected = 0.000000001;
    }
}

/**
 * Calculates resistors R1 and R2.
 * Used global variables: vsmin, vsmax, ipsmax, vgs_pwr_sh, vgstyp, v_overhead, vout, iout, rshunt, rdson.
 * This function does not change any global variable.
 * @return an array with value of R1 at index 0 AND value of R2 at index 1
 */
function calculateResistors()
{
    let numRows = 20, numSeries = 50;

    let arrayIPS = create2DArray(numRows + 1, numSeries + 1);
    let arrayIOUTS = create2DArray(numRows + 1, numSeries + 1);
    let arrayPTLD = create2DArray(numRows + 1, numSeries + 1);

    let max_r1 = 0, max_r2 = 0, max_ptld = new Array(numSeries + 1);

    for (let row = 0; row <= numRows; row++) // numRows rows / linear interpolation of VS range
    {
        let vs_temp = vsmin + ((vsmax - vsmin) / numRows) * row;

        for (let series = 1; series <= numSeries; series++) // numSeries series / linear interpolation of IPSmax range
        {
            let ipsmax_temp = (ipsmax / numSeries) * series;
            let r2_temp = (vgs_pwr_sh - vgstyp) / ipsmax_temp;
            let r1_temp = (vsmax + v_overhead - vout - Math.max(iout, ipsmax) * rshunt - ipsmax_temp * (r2_temp + rdson)) / ipsmax_temp;

            arrayIPS[row][series] = vs_temp - vout < vgstyp ? 0 : (vs_temp - vout <= vgs_pwr_sh ? Math.min((vs_temp - vout - vgstyp) / r2_temp, (vs_temp - vout) / (r2_temp + r1_temp + rdson)) : Math.min((vgs_pwr_sh - vgstyp) / r2_temp, (vs_temp - vout) / (r2_temp + r1_temp + rdson) ) );
            arrayIOUTS[row][series] = iout - arrayIPS[row][series] < 0 ? 0 : iout - arrayIPS[row][series];
            arrayPTLD[row][series] = (vs_temp - vout - rshunt * Math.max(iout, arrayIPS[row][series])) * arrayIOUTS[row][series] + rshunt * Math.max(iout, arrayIPS[row][series]) * Math.max(iout, arrayIPS[row][series]);

            if (!(arrayPTLD[row][series] < max_ptld[series])) {
                max_ptld[series] = arrayPTLD[row][series];
            }

            if (row == numRows) { // means this is the last row iteration, after we have collected all max_ptld values from all rows and for all series until the current one.
                // Acceptable recommended resistor values are in the series which 
                if (max_ptld[series] <= pvmax) {
                    if (max_r1 <= r1_temp && max_r2 <= r2_temp)
                    {
                        max_r1 = r1_temp;
                        max_r2 = r2_temp;
                    }
                }
            }
        }
    }

    return [max_r1, max_r2];
}

/**
 * Generates power and current data by interpolating the VS range.
 * Used global variables: vsmin, vsmax, vout, vgstyp, vgs_pwr_sh, r1_selected, r2_selected, rdson, iout, rshunt.
 * No global variables will be changed inside the function.
 * @return a 2D array with following data starting from index [X][0] to [X][7]: VS (interpolated range), IPS, IOUTS, IOUT, PTLD, PR1, PR2, X,
 * where X is a value in range from 0 to number of interpolations - 1.
 */
function generatePowerData()
{
    let m = 21, n = 8;
    let dataArray = create2DArray(m,n);

    for (let i = 0; i < m; i++)
    {
        let starting_point = Math.max(vsmin, vout + vps_cc);
        
        // V_S - Supply voltage
        dataArray[i][0] = (starting_point + (vsmax - starting_point) / 20 * i);

        // I_PS - Power shift current
        dataArray[i][1] = (parseFloat(dataArray[i][0]) - vout < vgstyp ? 0 : (parseFloat(dataArray[i][0]) - vout < vgs_pwr_sh ? Math.min( (parseFloat(dataArray[i][0]) - vout - vgstyp) / r2_selected, (parseFloat(dataArray[i][0]) - vout) / (r1_selected + r2_selected + rdson)) : Math.min( (vgs_pwr_sh - vgstyp) / r2_selected, (parseFloat(dataArray[i][0]) - vout) / (r1_selected + r2_selected + rdson)) ) );

        // I_OUTS - Internal current
        dataArray[i][2] = ( iout - parseFloat(dataArray[i][1]) < 0 ? 0 : iout - parseFloat(dataArray[i][1]) );

        // I_OUT - Output current
        dataArray[i][3] = ( parseFloat(dataArray[i][1]) + parseFloat(dataArray[i][2]) );

        // P_TLD1114 - Power on TLD1114
        dataArray[i][4] = ( parseFloat(dataArray[i][0]) - vout - rshunt * parseFloat(dataArray[i][3]) ) * parseFloat(dataArray[i][2]) + rshunt * parseFloat(dataArray[i][3]) * parseFloat(dataArray[i][3]);
        
        // P_R_1 - Power on R_1
        dataArray[i][5] = r1_selected * parseFloat(dataArray[i][1]) * parseFloat(dataArray[i][1]);

        // P_R_2 - Power on R_2
        dataArray[i][6] = r2_selected * parseFloat(dataArray[i][1]) * parseFloat(dataArray[i][1]);

        // X - Index
        dataArray[i][7] = (i);
    }

    return dataArray;
}

/**
 * General function for starting all calculations.
 * Called after all variables have been fetched.
 * This function DOES change global variables.
 * Used global variables: {iout, vin_set, k1, k2, tjmax, tambient, rthja} ∪ {variables used in functions calculateResistors() and generatePowerData()}
 * Changed global variables: rset, cfg_setting, output_cfg, pvmax, rshunt, r1, r2, powerData.
 */
function calculateData()
{
    // Input-only-dependent data and NOT used for further calculation -> order NOT important
    rset = iout > mid_current ? vin_set / iout * k2 : vin_set / iout * k1;
    cfg_setting = iout > mid_current ? 'GND' : 'open';
    output_cfg = iout > mid_current ? 'OUTH' : 'OUTL';

    // Input-only-dependent data and IS used for further calculation -> order IS important
    pvmax = (tjmax - tambient) / rthja;
    rshunt = iout > mid_current ? 0.75 : 0.75 * 2;

    // Calculate recommended resistor values
    let resistors = calculateResistors();
    if (toolMOSFETVersion == WITH_MOSFET) {
        r1 = resistors[0];
        r2 = resistors[1];
    } else if (toolMOSFETVersion == NO_MOSFET) {
        r1 = resistors[0] + resistors[1];
        r2 = 0.000000001;
    }

    // If the checkbox is ticked use calculated/recommended values.
    if (useRecommendedResistorValues) {
        r1_selected = r1;
        r2_selected = r2;
    }

    // Calculate power data
    powerData = generatePowerData();

    p1114max = findColumnMaxValueIn2DArray(powerData, 4);
    pr1_max = findColumnMaxValueIn2DArray(powerData, 5);
    pr2_max = findColumnMaxValueIn2DArray(powerData, 6);
    iout_max = findColumnMaxValueIn2DArray(powerData, 3);

    p1114max_index = extractColumnFrom2DArray(powerData, 4).indexOf(p1114max);
    pr1_max_index = extractColumnFrom2DArray(powerData, 5).indexOf(pr1_max);
    pr2_max_index = extractColumnFrom2DArray(powerData, 6).indexOf(pr2_max);
    iout_max_index = extractColumnFrom2DArray(powerData, 3).indexOf(iout_max);

    // Fix the floating point error!
    iout_max = roundDecimals(iout_max, NDEC + 1);

    // Set overal result and result description based on the calculated powers
    if (p1114max / pvmax > 1.20) {
        result = 'fail';
        result_description = 'power too high';
    } else if (iout_max > iout) {
        result = 'fail';
        result_description = 'current too high'
    } else if (toolMOSFETVersion === NO_MOSFET && r1_selected < 75.00) {
        result = 'R<sub>1</sub> < safe value';
        result_description = '<span data-toggle="modal" data-target="#pseudocodeModal" style="cursor: pointer;">see calculation steps</span>';
    } else if (p1114max / pvmax > 1.00) {
        result = 'possible fail';
        result_description = 'power high';
    } else {
        result = 'pass';
        result_description = 'OK';
    }

}

/**
 * Writes from the function calculateData() changed global variables into corresponding elements inside the HTML page.
 * No global variables will be changed inside the function.
 * TODO: break the function updateChartData() into 2 functions: calculateChartData() and updateChartData.
 * calculateChartData() changes the chart1 and chart2 variables but calculateChartData() does not.
 */
function updateData()
{
    switchConfigurationVersion(iout > mid_current ? 'HIGH' : 'LOW', currentDeviceSVG);
    
    setVRCanvas(currentDeviceSVG, 'rset', toKilo(rset), NDEC);
    setVRCanvas(currentDeviceSVG, 'pvmax', pvmax, NDEC);
    setVRCanvas(currentDeviceSVG, 'rshunt', rshunt, NDEC);
    setVRCanvas(currentDeviceSVG, 'r1', r1, NDEC);
    setVRCanvas(currentDeviceSVG, 'r2', r2, NDEC);
    
    setV('cfg_setting', cfg_setting);
    setV('output_cfg', output_cfg);

    setVR('pr1_max', pr1_max, NDEC);
    setVR('pr2_max', pr2_max, NDEC);
    setVR('p1114max', p1114max, NDEC);

    if (chart1 != null && chart2 != null) 
    {
        updateChartData(chart1, new Array(extractColumnFrom2DArray(powerData, 0), multiplyArrayByScalar(extractColumnFrom2DArray(powerData, 1), 1000), multiplyArrayByScalar(extractColumnFrom2DArray(powerData, 2), 1000), multiplyArrayByScalar(extractColumnFrom2DArray(powerData, 3), 1000)));
        updateChartData(chart2, new Array(extractColumnFrom2DArray(powerData, 0), extractColumnFrom2DArray(powerData, 4), extractColumnFrom2DArray(powerData, 5), extractColumnFrom2DArray(powerData, 6)));
    }

    let result_html = document.getElementById('result');
    let description_html = document.getElementById('result_description');

    result_html.innerHTML = result;
    description_html.innerHTML = result_description;

    // Set result class (visual change): normal, warning, error. If error then the charts and their legends will be grayed out.
    if (p1114max / pvmax > 1.20) {
        $(result_html).addClass('error'); $(result_html).removeClass('warning');
        $(description_html).addClass('error'); $(description_html).removeClass('warning');
        $('#chart-container').addClass('grayout');
        setV('p1114max', 'N/A');
    } else if (iout_max > iout) {
        $(result_html).addClass('error'); $(result_html).removeClass('warning');
        $(description_html).addClass('error'); $(description_html).removeClass('warning');
        $('#chart-container').addClass('grayout');
        setV('p1114max', 'N/A');
    } else if (toolMOSFETVersion === NO_MOSFET && r1_selected < 75.00) {
        $(result_html).removeClass('error'); $(result_html).addClass('warning');
        $(description_html).removeClass('error'); $(description_html).addClass('warning');
        $('#chart-container').removeClass('grayout');
    } else if (p1114max / pvmax > 1.00) {
        $(result_html).removeClass('error'); $(result_html).addClass('warning');
        $(description_html).removeClass('error'); $(description_html).addClass('warning');
        $('#chart-container').removeClass('grayout');
    } else {
        $(result_html).removeClass('error'); $(result_html).removeClass('warning');
        $(description_html).removeClass('error'); $(description_html).removeClass('warning');
        $('#chart-container').removeClass('grayout');
    }

    // Update TINA link with newly calculated parameters.
    let tina_params = '';
    let tina_link = 'https://design.infineon.com/tinademo/designer.php?path=EXAMPLESROOT%7CINFINEON%7CApplications%7CLighting%7CLED%20Lighting%7C&file=atv_12v_LED_lighting_driver_TLD1114-1EP_application_circuit_V1.tsc';

    if (toolMOSFETVersion === WITH_MOSFET) {
        tina_params = '&act=set:R_IN_SET:' + rset + '|set:RD:' + r1_selected + '|set:RS:' + r2_selected;
        document.getElementById('tina_simulator').setAttribute('data-externLink', tina_link + encodeURI(tina_params));
    } else if (toolMOSFETVersion === NO_MOSFET) {
        tina_link = 'https://design.infineon.com/tinademo/designer.php?path=EXAMPLESROOT%7CINFINEON%7CApplications%7CLighting%7CLED%20Lighting%7C&file=atv_12v_LED_lighting_driver_TLD1114-1EP_no_mosfet.tsc';
        tina_params = '&act=set:R_IN_SET:' + rset + '|set:RD:' + r1_selected;
        document.getElementById('tina_simulator').setAttribute('data-externLink', tina_link + encodeURI(tina_params));
    }

    updatePseudoCode();
}

/**
 * Executed each time any input field inside the SVG schematic has changed. Starts (re)calculation.
 */
function onUpdate(input_field_id)
{
    last_updated_param_id = input_field_id != null ? input_field_id : last_updated_param_id;

    fetchData();
    calculateData();
    updateData();
}

/**
 * Initiate first calculation after everything is ready.
 */
function onPageFullyLoaded()
{
    fetchData();
    calculateData();

    // If first run MF or NonMF --> Use recommended resistor values (and set the values to the schematic visually).
    if (firstRunMF) {
        param_map.get('r1_selected').value = roundDecimals( r1, NDEC );
        param_map.get('r2_selected').value = roundDecimals( r2, NDEC );
        setVRCanvas(currentDeviceSVG, 'r1_selected', r1, NDEC);
        setVRCanvas(currentDeviceSVG, 'r2_selected', r2, NDEC);
        fetchData();
        calculateData();
        firstRunMF = false;
    } else if (firstRunNonMF) {
        param_map.get('r1_selected').value = roundDecimals( r1, NDEC );
        setVRCanvas(currentDeviceSVG, 'r1_selected', r1, NDEC);
        fetchData();
        calculateData();
        firstRunNonMF = false;
    }

    updateData();

    // Destroy existing chart instances before creating new ones.
    if (chart1 != null && chart2 != null) {
        chart1.destroy();
        chart2.destroy();
    }
    
    // Create new chart instances for new calculation.
    chart1 = createChart1(canvas1);
    chart2 = createChart2(canvas2);

    // Disable P_R_2 if currently selected tool version is "Without MOSFET".
    chart2.data.datasets[2].hidden = toolMOSFETVersion == NO_MOSFET;
    
    updateData();
}

/**
 * Update values of parameters from the formula in HTML.
 */
function updatePseudoCode() {
    // Update result parameters in the map for later use with the function updateFormulas(param_map)          
    updatePseudoParamValue('tjmax', tjmax);
    updatePseudoParamValue('tambient', tambient);
    updatePseudoParamValue('rthja', rthja);
    updatePseudoParamValue('pvmax', pvmax);
    updatePseudoParamValue('k_used', iout > mid_current ? 2 : 1);
    updatePseudoParamValue('vin_set', vin_set);
    updatePseudoParamValue('iout', iout * 1000);
    updatePseudoParamValue('iout_max', iout_max * 1000);
    updatePseudoParamValue('iout_max_index', iout_max_index);
    updatePseudoParamValue('mid_current', mid_current * 1000);
    updatePseudoParamValue('k_value', iout > mid_current ? k2 : k1);
    updatePseudoParamValue('rset', rset / 1000);
    updatePseudoParamValue('r1_selected', r1); // R_1 recommended
    updatePseudoParamValue('r1', r1_selected); // R_1 selected
    updatePseudoParamValue('r2', r2_selected); // R_2 selected
    updatePseudoParamValue('r1r', useRecommendedResistorValues ? r1_selected : ''); // R_1 recommended
    updatePseudoParamValue('r2r', useRecommendedResistorValues ? (toolMOSFETVersion === NO_MOSFET ? '' : r2_selected) : ''); // R_2 recommended
    updatePseudoParamValue('r1i', useRecommendedResistorValues ? '' : r1_selected); // R_1 input
    updatePseudoParamValue('r2i', useRecommendedResistorValues ? '' : (toolMOSFETVersion === WITH_MOSFET ? r2_selected : '')); // R_2 input
    updatePseudoParamValue('result', result);
    updatePseudoParamValue('result_description', result_description);
    updatePseudoParamValue('cfg_setting', cfg_setting);
    updatePseudoParamValue('output_cfg', output_cfg);
    updatePseudoParamValue('pr1_max', pr1_max);
    updatePseudoParamValue('pr1_max_index', pr1_max_index);
    updatePseudoParamValue('pr2_max', pr2_max);
    updatePseudoParamValue('pr2_max_index', pr2_max_index);
    updatePseudoParamValue('p1114max', p1114max);
    updatePseudoParamValue('p1114max_index', p1114max_index);
    updatePseudoParamValue('vsmin', vsmin);
    updatePseudoParamValue('vsmax', vsmax);
    updatePseudoParamValue('vps_cc', vps_cc);
    updatePseudoParamValue('vout', vout);
    updatePseudoParamValue('vs_range_min', roundDecimals(Math.max(vsmin, vout + vps_cc), NDEC));
    updatePseudoParamValue('vs_range_max', vsmax);
    updatePseudoParamValue('v_overhead', v_overhead);
    updatePseudoParamValue('rdson', rdson);
    updatePseudoParamValue('v_overhead', v_overhead);
    updatePseudoParamValue('vin_set', vin_set);
    updatePseudoParamValue('vgs_pwr_sh', vgs_pwr_sh);
    updatePseudoParamValue('rshunt', rshunt);
    updatePseudoParamValue('vgstyp', toolMOSFETVersion === WITH_MOSFET ? vgstyp : '');
    updatePseudoParamValue('rdson', toolMOSFETVersion === WITH_MOSFET ? rdson : '');
    updatePseudoParamValueImmediately('ips_case', toolMOSFETVersion == NO_MOSFET ? 0 : 1);
    updatePseudoParamValue('r1_warn', toolMOSFETVersion === NO_MOSFET && r1_selected < 75 ? 1 : 0);

    updateFormulas();
}
