/**
 * Create an empty 2D array with specified number of rows and columns.
 */
function create2DArray(rows, cols)
{
	let array = new Array(rows);
	for(let i = 0; i < rows; i++)
		for(let j = 0; j < cols; j++)
			array[i] = new Array(cols);
    return array;
}

/**
 * Extract single column from 2D array.
 */
function extractColumnFrom2DArray(array2d, column_id)
{
    if (column_id < 0) return undefined;
    if (column_id >= array2d.length) return undefined;

	let return_array = new Array(array2d.length);

	for (let i = 0; i < array2d.length; i++) {
		return_array[i] = array2d[i][column_id];
	}

	return return_array;
}

/**
 * Multiply each element of a one dimensional array with a scalar and return the product array.
 */
function multiplyArrayByScalar(numberArray, scalar) {

    return_array = new Array(numberArray.length);

    for (let i = 0; i < numberArray.length; i++) {
        return_array[i] = numberArray[i] * scalar;
    }

    return return_array;
}

/**
 * Update specified chart with specified data.
 */
function updateChartData(chart, data)
{
	chart.data.labels = toFixedNumberArray(data[0], 2);

	for (let i = 1; i < data.length; i++) {
		chart.data.datasets[i-1].data = toFixedNumberArray(data[i], 3);
	}
	
	chart.update();
}

/**
 * Returns rounded array of numbers by specified number of decimal places.
 */
function toFixedNumberArray(numberArray, decimalPoints)
{
	return_array = new Array(numberArray.length);

	for (let i = 0; i < numberArray.length; i++) {
		try {
			return_array[i] = numberArray[i].toFixed(decimalPoints);
		} catch (err) {
			return_array[i] = numberArray[i];
		}
	}

	return return_array;
}

/**
 * Returns rounded number by specified number of decimal places.
 */
function roundDecimals(number, ndecimals)
{
    return parseFloat(parseFloat(number).toFixed(ndecimals));
}

/**
 * Find max value inside column with specified index inside 2D array.
 */
function findColumnMaxValueIn2DArray(array2d, column_index) {
    let maxValue = array2d[0][column_index];

    for (let i = array2d.length - 1; i >= 0; i--) {
        maxValue = maxValue < array2d[i][column_index] ? array2d[i][column_index] : maxValue;
    }

    return maxValue;
}

/**
 * Set value of specified input element.
 */
function setV(field_name, _value) {

    if (document.getElementById(field_name).tagName.toLowerCase() == 'input')
        document.getElementById(field_name).value = _value;
    else
        document.getElementById(field_name).innerHTML = _value;
}

/**
 * Set numeric value of specified input element (rounded by number of decimal places defined globally).
 */
function setVR(field_name, _value, ndecimals) {
    if (document.getElementById(field_name).tagName.toLowerCase() == 'input')
        document.getElementById(field_name).value = roundDecimals(_value, ndecimals);
    else
        document.getElementById(field_name).innerHTML = roundDecimals(_value, ndecimals);
}

/**
 * Converts number from mili to base.
 */
function fromMili(number) {
    return number / 1000;
}

/**
 * Converts number from base to mili.
 */
function toMili(number) {
    return number * 1000;
}

/**
 * Converts number from kilo to base.
 */
function fromKilo(number) {
    return number * 1000;
}

/**
 * Converts number from base to kilo.
 */
function toKilo(number) {
    return number / 1000;
}

/**
 * Creates a chart inside specified canvas. This function is hardcoded and should be (re)moved somewhere else.
 */
function createChart1(canvas1)
{
    let chart1 = new Chart(canvas1, 
    {
        type: 'line',
        data: 
        {
            datasets: [
                {
                    label: 'Power shift current: IPS',
                    backgroundColor: '#a5c9be',
                    borderColor: '#a5c9be',
                    // data: extractColumnFrom2DArray(powerData, 1),
                    fill: false,
                },
                {
                    label: 'Internal current: IOUTS',
                    backgroundColor: '#b6c676',
                    borderColor: '#b6c676',
                    // data: extractColumnFrom2DArray(powerData, 2),
                    fill: false,
                },
                {
                    label: 'Output current: IOUT',
                    backgroundColor: '#f18c4f',
                    borderColor: '#f18c4f',
                    // data: extractColumnFrom2DArray(powerData, 3),
                    fill: false,
                },
            ],
            // labels: extractColumnFrom2DArray(powerData, 0),
        },
        options:
        {
            legend:
            {
                display: false,
            },
            responsive: true,
            maintainAspectRatio: false,
            tooltips: 
            {
                mode: 'index',
                intersect: false
            },
            hover: 
            {
                mode: 'nearest',
                intersect: true
            },
            scales: 
            {
                xAxes:
                [
                    {
                        display: true,
                        scaleLabel:
                        {
                            display: true,
                            labelString: 'Supply Voltage (V)'
                        }
                    }
                ],
                yAxes:
                [
                    {
                        display: true,
                        scaleLabel:
                        {
                            display: true,
                            labelString: 'Current (mA)'
                        }
                    }
                ]
            },
        },
    });

    return chart1;
}


/**
 * Creates a chart inside specified canvas. This function is hardcoded and should be (re)moved somewhere else.
 */
function createChart2(canvas2)
{
    let chart2 = new Chart(canvas2, 
    {
        type: 'line',
        data: 
        {
            datasets: [
                {
                    label: 'PTLD1114',
                    backgroundColor: '#a5c9be',
                    borderColor: '#a5c9be',
                    // data: extractColumnFrom2DArray(powerData, 4),
                    fill: false,
                },
                {
                    label: 'PR1',
                    backgroundColor: '#b6c676',
                    borderColor: '#b6c676',
                    // data: extractColumnFrom2DArray(powerData, 5),
                    fill: false,
                },
                {
                    label: 'PR2',
                    backgroundColor: '#f18c4f',
                    borderColor: '#f18c4f',
                    // data: extractColumnFrom2DArray(powerData, 6),
                    fill: false,
                },
            ],
            // labels: extractColumnFrom2DArray(powerData, 0),
        },
        options:
        {
            legend:
            {
                display: false,
            },
            responsive: true,
            maintainAspectRatio: false,
            tooltips: 
            {
                mode: 'index',
                intersect: false
            },
            hover: 
            {
                mode: 'nearest',
                intersect: true
            },
            scales: 
            {
                xAxes:
                [
                    {
                        display: true,
                        scaleLabel:
                        {
                            display: true,
                            labelString: 'Supply Voltage (V)'
                        }
                    }
                ],
                yAxes:
                [
                    {
                        display: true,
                        scaleLabel:
                        {
                            display: true,
                            labelString: 'Power loss (W)'
                        }
                    }
                ]
            },
        },
    });
    
    return chart2;
}

/**
 * The goal of this function is to set correct minimum and maximum values of input parameters so they cannot accept invalid values.
 * WARNING: Ranges of all dependent parameters must be already set to correct values before calling this function.
 */
function guardDependentParameters(lower_param_id, higher_param_id, last_updated_param_id)
{
    if (lower_param_id === last_updated_param_id) {
        let lower_param = param_map.get(lower_param_id), higher_param = param_map.get(higher_param_id);
        higher_param.min = lower_param.value;
    } else if (higher_param_id === last_updated_param_id) {
        let lower_param = param_map.get(lower_param_id), higher_param = param_map.get(higher_param_id);
        lower_param.max = higher_param.value;
    }
}

/**
 * TODO: Update description!
 * Does the same kind of job like guardDependentParameters() except this function limits only the lower parameter.
 * WARNING: Ranges of all dependent parameters must be already set to correct values before calling this function.
 */
function guardDependentParametersLowerFor(lower_param_id, higher_param_id, for_param_id, last_updated_param_id)
{
    if (higher_param_id === last_updated_param_id) {
        param_map.get(lower_param_id).max = param_map.get(higher_param_id).value - param_map.get(for_param_id).value;
    }
}

/**
 * Used to switch between RDSON and non-RDSON version of the tool.
 * Parameter x used to decide which tool to use: 1 - MOSFET version, 2 - non-MOSFET version.
 * Variable toolMOSFETVersion defined inside calculation.js.
 */
function switchVersionRDSON(x) {

    // Do nothing if the tool version didn't change.
    if(x == toolMOSFETVersion) {
        return;
    }

    // Handle recommended resistor values state and checkbox: Save and apply states between version switchings.
    // According to the new value of useRecommendedResistorValues the tick should be applied after a new SVG schematic loads.
    // This is because the checkbox is located in the SVG image itself and not in the HTML.
    // The checkbox will be updated visually only in updateResistorCheckbox() each time an SVG fully loads according to value of useRecommendedResistorValues.
    if (x == NO_MOSFET) { 
        recommendedMF = useRecommendedResistorValues; // Back-up current value of useRecommendedResistorValues for WITH_MOSFET version.
        useRecommendedResistorValues = recommendedNonMF; // Set backed-up value to NO_MOSFET version.
    } else if (x == WITH_MOSFET) {
        recommendedNonMF = useRecommendedResistorValues; // Back-up current value of useRecommendedResistorValues for NO_MOSFET version.
        useRecommendedResistorValues = recommendedMF; // Set backed-up value to WITH_MOSFET version.
    }

    toolMOSFETVersion = x;

    let currentDataURL = document.getElementById("schematic").data;
    let newDataURL = 'images/' + (toolMOSFETVersion == WITH_MOSFET ? 'TLD1114_MOSFET_OUTH_OUTL' : 'TLD1114_no_MOSFET_OUTH_OUTL') + ".svg";

    if (!currentDataURL.endsWith(newDataURL)) {
        document.getElementById("schematic").data = newDataURL;
    }
    
   if (toolMOSFETVersion == WITH_MOSFET) {
       /* Revert chart field PR2 */
       chart2.data.datasets[2].label = "PR2";
       chart2.data.datasets[2].backgroundColor = '#32CD32';
       chart2.data.datasets[2].borderColor = '#32CD32';
       chart2.update();
    
    } else if (toolMOSFETVersion == NO_MOSFET) {
       /* Remove chart field PR2 */
       chart2.data.datasets[2].label = "";
       chart2.data.datasets[2].data = null;
       chart2.data.datasets[2].backgroundColor = "rgba(0, 0, 0, 0)";
       chart2.data.datasets[2].borderColor = "rgba(0, 0, 0, 0)";
       chart2.update();
    }

    /* Trigger the resize event to update the GUI after some elements have been added/removed. */
    setTimeout(function(){ 
        chart1.resize();
        chart2.resize();
     }, 0);
     $(window).trigger('resize');
}


/**
 * Used to switch between OUTH and OUTL configuration versions of the schematics (both MOSFET and non-MOSFET) inside the canvas.
 * Expects LOW or HIGH as input parameter x.
 */
function switchConfigurationVersion(x, canvas) {

    let group_outL = canvas.getElementById('group_OUTL_OPEN');
    let group_outH = canvas.getElementById('group_OUTH_GND');

    if( x == 'LOW' ) {
        $(group_outL).show();
        $(group_outH).hide();
    } else if ( x == 'HIGH' ) {
        $(group_outL).hide();
        $(group_outH).show();
    }
}

/**
 * Listens on page fully loaded event. Put inside what should be done after page fully loads.
 */
window.addEventListener('DOMContentLoaded', (event) => {

    // No need to load GUI elements while testing.
    if (test_environment) {
        return;
    }

    /* Load external html parts. */
    loadEmbeddedAll();

    /* Set tool version - get it from the toolbox */
    document.getElementById('releaseNotes').innerHTML = getToolboxVersion();

    // Add listeners to chart legend items.
    addChartLegendListeners();

    /**
     * Add listener for schematic selection tab.
     */
    $("#schematic-tab-holder > table > tbody > tr > td").click(function(e) {
        $(this).addClass("selected").siblings().removeClass("selected");

        let index = $(this).index();

        $("#schematics-container").children().eq(index).removeClass("hidden").siblings().addClass("hidden");

        switchVersionRDSON(index + 1);

        // Disable gray-out effect on all chart legend items
        for(let i = 1; i < 7; i++) {
            let elem = document.getElementById('legend_data' + i);
            $(elem).children(".rectangle").first().removeClass("hidden");
        }
    });

    /**
     * Add listener for chart selection tab.
     */
    $("#chart-tab-holder > table > tbody > tr > td").click(function(e) {
        $(this).addClass("selected").siblings().removeClass("selected");

        let index = $(this).index();

        $("#chart-container").children().eq(index).removeClass("hidden").siblings().addClass("hidden");

        if (index === 0) {
            chart1.resize();
        } else if (index === 1) {
            chart2.resize();
        }
    });

    /**
     * Reload default input values on element click.
     */
    $('#reload_btn').click(function(e) {

        if (toolMOSFETVersion === WITH_MOSFET) {
            param_map = cloneParamMap(PARAM_MAP);
            param_map_MF_temp = param_map;
            firstRunMF = true;
        } else if (toolMOSFETVersion === NO_MOSFET) {
            param_map = cloneParamMap(PARAM_MAP);
            param_map_nMF_temp = param_map;
            firstRunNonMF = true;
        }

        document.getElementById("schematic").data = document.getElementById("schematic").data;

        // Disable gray-out effect on all chart legend items
        for(let i = 1; i < 7; i++) {
            let elem = document.getElementById('legend_data' + i);
            $(elem).children(".rectangle").first().removeClass("hidden");
        }
    });
});

/** 
 * Add onclick event listener to all external links to indicate that they open in user's default browser.
 */
function addLinkTooltips() {
    let exlink_elements = Array.from(document.getElementsByClassName("extern-link"));
    exlink_elements.forEach(function(el) {
        const extippy = tippy(el, {
            content: 'The link will now open in your default web browser.',
            trigger: 'manual',
            placement: 'right',
            arrow: true,
            animation: 'scale',
            theme: 'berry',
            offset: '0, 5',
        });

        el.addEventListener("click", function() {
            extippy.show(300);
            setTimeout(function(){ extippy.hide(300); }, 1500);
        });
    });
}

/**
 * Load all dynamically embedded HTML elements. In some browser this will not work due to Cross Origin limitations.
 * After one embedded element has been loaded its parent wrapper will be removed.
 */
function loadEmbeddedAll() {
    $(function () {
        let includes = $('[data-cms-include]');
        let count = 0;
        jQuery.each(includes, function (index, element) {
            let file = $(this).data('cms-include');
            $(this).load('parts/' + file, null, function() {
                $(element.firstElementChild).unwrap();

                /* Add tooltips to links only after all link elements have been loaded. */
                if (++count === includes.length) {
                    addLinkTooltips();
                    // tippy(document.getElementById('screenshot'), {content: 'Make a screenshot', theme: 'berry'});
                    tippy(document.getElementById('pseudocode'), {content: 'Show calculation steps', theme: 'berry'});
                    tippy(document.getElementById('aboutImg'), {content: 'About the tool', theme: 'berry'});

                    /* Trigger onclick event on an element to notify JxBrowser that content has been fully loaded. */
                    document.getElementById('page-fully-loaded-click-notifier').click();
                }
            });
        });
    });
}

/**
 * Called when svg schematic image changes. It will add onClick listeners to each input parameter's label.
 * This is the only function which is called directly from HTML on load event, after an SVG has fully loaded.
 * Consider calling this function from JavaScript listener (to make HTML free from JavaScript).
 * Concern: JavaScript code must be loaded before the HTML in that case.
 * To research: Do browsers load JavaScript and HTML synchronously or asynchronously?
 */
function addInputListeners() {
    
    // Maintain copies of parameters between tab switchings (MOSFEET vs. non-MOSFET).
    if (toolMOSFETVersion === WITH_MOSFET) {
        if (param_map_MF_temp !== null) {
            param_map = param_map_MF_temp;
        } else {
            param_map = cloneParamMap(PARAM_MAP);
            param_map_MF_temp = param_map;
        }
    } else if (toolMOSFETVersion === NO_MOSFET) {
        if (param_map_nMF_temp !== null) {
            param_map = param_map_nMF_temp;
        } else {
            param_map = cloneParamMap(PARAM_MAP);
            param_map_nMF_temp = param_map;
        }
    }

    // Update reference to currently used svg because svg images can be switched.
    currentDeviceSVG = document.getElementById('schematic').contentDocument;

    /* Find elements which start with '#' prefix sign and remove the sign. */
    Array.from(currentDeviceSVG.querySelectorAll("g > g > desc")).forEach(function(el) {
        if (el.textContent.startsWith('#')) {
            el.parentNode.querySelector('text').id = el.textContent.substring(1);
        }
    });

    /* Remove all title elements to get rid of random tooltips. */
    Array.from(currentDeviceSVG.querySelectorAll("title")).forEach(function(el) {
        el.parentNode.removeChild(el);
    });

    // Make svg's elements non-selectable.
    let sheet = currentDeviceSVG.styleSheets[0];
    sheet.insertRule('svg {-webkit-touch-callout: none;-webkit-user-select: none;-khtml-user-select: none;-moz-user-select: none;-ms-user-select: none;user-select: none;}', 0);
    sheet.insertRule('.parameter { cursor: pointer; }', 0);

    let input_ids = '#rdson, #vgstyp, #vout, #iout, #vsmin, #vsmax, #tambient, #rthja, #v_overhead, #r1_selected, #r2_selected';

    let input_elements = Array.from(currentDeviceSVG.querySelectorAll(input_ids));

    input_elements.forEach(function(el) {
        el.parentNode.parentNode.id = '_' + el.id;
        el.parentNode.parentNode.addEventListener("click", click); // Add onClickListener to each input parameter label.
        el.parentNode.parentNode.classList.add('parameter');

        let param = param_map.get(el.id);

        /* Set content of parameter text field (initially). */
        el.textContent = "" + param.value + " " + param.unit;
    });

    function click(e) {
        let text_element = currentDeviceSVG.getElementById(this.id.substring(1));
        openInputBoxFor(text_element);
    }

    // Add an event listener to the checkbox of the newly loaded SVG. 
    // Concern (memory/CPU): What happens to the event listener of the previous/removed/destroyed SVG?
    currentDeviceSVG.getElementById('resistorsCheckBox').addEventListener("click", function() {
        toggleResistorMode();
    });

    // Update state of the resistors checkbox tick according to updateResistorCheckbox's value.
    updateResistorCheckbox();

    onPageFullyLoaded(); // Start the calculation after the image has loaded. Call this function only when everything else is ready (fully loaded).
}

/**
 * Called when clicked on an input parameter inside svg element. It will create an input element and show it on top of the parameter label.
 * After the parameter has been edited (onchange event) it is going to call onUpdate(...) method.
 */
function openInputBoxFor(input_field) {

    let box_position = input_field.parentNode.parentNode.getBoundingClientRect();
    let x = box_position.x + 20 - 0;
    let y = box_position.y + 20 - 2;
    let w = input_field.getBoundingClientRect().x + input_field.getComputedTextLength() - box_position.x + 32;
    w = w < 120 ? 120 : w;
    let h = box_position.height + 2;
    let tippy_instance;

    let param = param_map.get(input_field.id);

    let input_box = document.createElement('input');
    let input_div = document.createElement('div');
    let input_unit = document.createElement('div');
    let input_unit_inner = document.createElement('span');

    input_div.classList.add('input-group');
    input_box.classList.add('form-control');
    input_unit.classList.add('input-group-append');

    input_div.classList.add('valid');

    input_field.style.display = 'none';
    input_box.setAttribute('data-for', input_field.id);
    input_box.setAttribute('type', 'number');
    input_box.setAttribute('title', ' ');
    input_box.setAttribute('min', param.min);
    input_box.setAttribute('value', param.value);
    input_box.setAttribute('max', param.max);
    input_box.setAttribute('step', typeof param.step !== 'undefined' ? param.step : '');
    input_div.setAttribute('style', 'position: absolute; width: ' + w + 'px; height: ' + h + 'px; top: ' + y + 'px; left: ' + x + 'px;');
    input_box.setAttribute('style', 'height: ' + h + 'px; outline: none !important; border-radius: 4px 0px 0px 4px;');
    input_unit.setAttribute('style', 'height: ' + h + 'px;');

    input_unit_inner.setAttribute('style', 'height: inherit;');
    input_unit_inner.classList.add('input-group-text');
    input_unit_inner.innerHTML = param.unit;

    input_unit.appendChild(input_unit_inner);
    input_div.appendChild(input_box);
    input_div.appendChild(input_unit);

    
    window.addEventListener("resize", focusout);
    input_box.addEventListener("focusout", focusout);
    input_box.addEventListener("input", input);
    input_box.addEventListener("keypress", keypress);
    

    function focusout(e) {
        /* Remove all associated listeners! */
        window.removeEventListener("resize", focusout);
        input_box.removeEventListener("focusout", focusout);
        input_box.removeEventListener("input", input);
        input_box.removeEventListener("keypress", keypress);

        /* Remove the input text box itself. */
        input_div.parentNode.removeChild(input_div);
        tippy_instance.destroy();

        /* Update the parameter inside svg only after the box has been closed! */
        input_field.textContent = "" + roundDecimals(param.value, NDEC) + " " + param.unit;
        input_field.style.display = 'initial';
    }

    function input(e) {
        if (isNumberInRange(this.value, param.min, param.max)) {
            $(input_div).removeClass("invalid").addClass("valid").siblings();
        } else {
            $(input_div).removeClass("valid").addClass("invalid").siblings();
        }
    }

    function keypress(event) {

        /* Forbid non-integer values if parameter is integer only. */
        if (param.type == 'integer' && !isNumericInput(event)) {
            event.preventDefault();
        }

        /* Close the input box when "Enter" key has been clicked on, if the entered number is in allowed range. */
        if (event.keyCode == 13 && isNumberInRange(this.value, param.min, param.max)) {

            param.value = parseFloat(input_box.value);

            onUpdate(input_field.id);

            input_box.blur();
        }
    }

    document.getElementById('schematic').parentNode.appendChild(input_div);

    let tippy_content = 
    '<div style="display: flex;"> <small style=""> <i>Value:</i> </small> <span style=" text-align: left;"></div> <div style="display: flex; margin-bottom: 3px;"> ' + param.value + ' ' + param.unit + ' </span> </div>' +
    '<div style="display: flex; justify-content: space-between;"> <small style=""> <i>Min:</i> </small> <small style=" text-align: right; padding-left: 70px;"> <i>Max:</i> </small> </div>' +
	'<div style="display: flex; justify-content: space-between;"> <span style="float: left;">' + param.min + ' ' + param.unit + '</span> <span style=" text-align: right; float: right; padding-left: 20px;">' + param.max + ' ' + param.unit + '</span> </div>';

    if (input_field.id == 'r1_selected') {
        tippy_content = tippy_content + '<hr style="padding: 0px; margin: 0px; margin-bottom: 3px; margin-top: 1px; border: 0.25px solid rgba(255, 255, 255, .1);"> <small style="float: left; padding-top: 2.2px;"><b><i>Recommended:</i></b></small><span style="float: left; padding-left: 10px;">' + roundDecimals(r1, NDEC) + ' ' + param.unit + '</span><br>';
    } else if (input_field.id == 'r2_selected') {
        tippy_content = tippy_content + '<hr style="padding: 0px; margin: 0px; margin-bottom: 3px; margin-top: 1px; border: 0.25px solid rgba(255, 255, 255, .1);"> <small style="float: left; padding-top: 2.2px;"><b><i>Recommended:</i></b></small><span style="float: left; padding-left: 10px;">' + roundDecimals(r2, NDEC) + ' ' + param.unit + '</span><br>';
    }


    tippy_instance = tippy(input_div, {
        theme: 'black',
        arrow: true,
        arrowType: 'round',
        animation: 'fade',
        duration: 200,
        flip: true,
        placement: 'right',
        flipBehavior: ["left", "right"],
        hideOnClick: true,
        content: tippy_content,
        maxWidth: 200,
    });

    input_box.focus();
    input_box.select();
}

/**
 * Returns true if <number> is between <minimum> and <maximum>.
 */
function isNumberInRange(number, minimum, maximum) {
    return minimum <= number && number <= maximum && number !== '';
}

/**
 * Get numeric value of specified input parameter from input parameters map.
 */
function getParamValue(param_id) {
    return parseFloat( param_map.get(param_id).value );
}

/**
 * Set numeric value of specified input element of a canvas (rounded by number of decimal places defined globally).
 */
function setVRCanvas(canvas, field_id, value, ndecimals) {
    let param = param_map.get(field_id);

    param.value = roundDecimals(value, ndecimals);

    // In case there are more elements with the same id (usually only output elements).
    Array.from(canvas.querySelectorAll('[id=' + field_id +']')).forEach(function(el) {
        el.textContent = param.value + ' ' + param.unit;
    });
}

/* Add listeners to chart legend elements to be able to show/hide chart data. */
function addChartLegendListeners() {
    document.getElementById("legend_data1").addEventListener("click", function() {
        chart1.data.datasets[0].hidden = !chart1.data.datasets[0].hidden;
        chart1.update();

        if (chart1.data.datasets[0].hidden) {
            $(this).children(".rectangle").first().addClass("hidden");
        } else {
            $(this).children(".rectangle").first().removeClass("hidden");
        }
    });
    document.getElementById("legend_data2").addEventListener("click", function() {
        chart1.data.datasets[1].hidden = !chart1.data.datasets[1].hidden;
        chart1.update();

        if (chart1.data.datasets[1].hidden) {
            $(this).children(".rectangle").first().addClass("hidden");
        } else {
            $(this).children(".rectangle").first().removeClass("hidden");
        }
    });
    document.getElementById("legend_data3").addEventListener("click", function() {
        chart1.data.datasets[2].hidden = !chart1.data.datasets[2].hidden;
        chart1.update();

        if (chart1.data.datasets[2].hidden) {
            $(this).children(".rectangle").first().addClass("hidden");
        } else {
            $(this).children(".rectangle").first().removeClass("hidden");
        }
    });
    document.getElementById("legend_data4").addEventListener("click", function() {
        chart2.data.datasets[0].hidden = !chart2.data.datasets[0].hidden;
        chart2.update();

        if (chart2.data.datasets[0].hidden) {
            $(this).children(".rectangle").first().addClass("hidden");
        } else {
            $(this).children(".rectangle").first().removeClass("hidden");
        }
    });
    document.getElementById("legend_data5").addEventListener("click", function() {
        chart2.data.datasets[1].hidden = !chart2.data.datasets[1].hidden;
        chart2.update();

        if (chart2.data.datasets[1].hidden) {
            $(this).children(".rectangle").first().addClass("hidden");
        } else {
            $(this).children(".rectangle").first().removeClass("hidden");
        }
    });
    document.getElementById("legend_data6").addEventListener("click", function() {
        chart2.data.datasets[2].hidden = !chart2.data.datasets[2].hidden;
        chart2.update();

        if (chart2.data.datasets[2].hidden) {
            $(this).children(".rectangle").first().addClass("hidden");
        } else {
            $(this).children(".rectangle").first().removeClass("hidden");
        }
    });
}

/**
 * Get version number from Infineon Toolbox. 
 * It is required that Infineon Toolbox makes the window.bundleVersion() available.
 */
function getToolboxVersion() {
    try {
        if (window.bundleVersion()) {
            let versionStr = window.bundleVersion();
            return versionStr;
        } else {
            return 'N/A';
        }
    } catch (error) {
        return 'N/A';
    }
}

/**
 * Used to toggle between recommended resistor mode and manual user input resistor mode.
 */
function toggleResistorMode() {
    let resistorsCheckBox = currentDeviceSVG.getElementById('resistorsCheckBox');
    let resistorsCheckBoxTick = currentDeviceSVG.getElementById('resistorsCheckBoxTick');
    let resistorGroupSelected = currentDeviceSVG.getElementById('resistorGroupSelected');
    let resistorGroupRecommended = currentDeviceSVG.getElementById('resistorGroupRecommended');

    if (resistorsCheckBox.dataset.ticked == 'true') {
        useRecommendedResistorValues = false;
        onUpdate('r1_selected'); onUpdate('r2_selected');
        resistorsCheckBox.dataset.ticked = 'false'
        $(resistorsCheckBoxTick).hide();
        $(resistorGroupSelected).show();
        $(resistorGroupRecommended).hide();
    } else {
        resistorsCheckBox.dataset.ticked = 'true'
        useRecommendedResistorValues = true;
        onUpdate('r1_selected'); onUpdate('r2_selected');
        $(resistorsCheckBoxTick).show();
        $(resistorGroupSelected).hide();
        $(resistorGroupRecommended).show();
    }
}

/** 
 * Tick or untick the recommended resistor values checkbox depending on useRecommendedResistorValues (global variable). Tick if true, untick if false.
 * To be called every time when SVG changes. The variable useRecommendedResistorValues can change in: toggleResistorMode() and switchVersionRDSON().
 */
function updateResistorCheckbox() {
    let resistorsCheckBox = currentDeviceSVG.getElementById('resistorsCheckBox');
    let resistorsCheckBoxTick = currentDeviceSVG.getElementById('resistorsCheckBoxTick');
    let resistorGroupSelected = currentDeviceSVG.getElementById('resistorGroupSelected');
    let resistorGroupRecommended = currentDeviceSVG.getElementById('resistorGroupRecommended');

    if (!useRecommendedResistorValues) {
        resistorsCheckBox.dataset.ticked = 'false'
        $(resistorsCheckBoxTick).hide();
        $(resistorGroupSelected).show();
        $(resistorGroupRecommended).hide();
    } else {
        resistorsCheckBox.dataset.ticked = 'true'
        $(resistorsCheckBoxTick).show();
        $(resistorGroupSelected).hide();
        $(resistorGroupRecommended).show();
    }
}

/* Open the w_url with default behaviour for PDF files: 1440x810, fit page by width, disable toolbar. */
function openPDFInNewWindow(w_url) {
    let pdf_window = window.open(w_url + '#view=fitH', '_blank', 'top=100,left=100,width=1440,height=810,titlebar=0,toolbar=0');
}
