var test_environment = true;

// ------------------------------------------------------------------------------ #1

QUnit.test( "Formula correctness test - Test Case #1", function( assert ) {
  // Set input values and product parameters
  toolMOSFETVersion = 1; useRecommendedResistorValues = false;
  vsmin = 8; vsmax = 16; vout = 6; iout = fromMili( 360 ); tambient = 85; rthja = 65; v_overhead = 5;
  rdson = 1; vgstyp = 2; r1_selected = 20.00; r2_selected = 8.10;
  vin_set = 1.22; k1 = 900; k2 = 890; ipsmax = fromMili( 500 ); vgs_pwr_sh = 5.50; vsmax_spec = 5.50;
  vsmin_spec = 40; iout_min = fromMili( 13.5 ); iout_max = fromMili( 360 ); vps_cc = 0.75; vout_cc = 1.40; tjmax = 150.00;

  // Start the calculation
  calculateData();

  // Set input values and product parameters
  toolMOSFETVersion = 1; useRecommendedResistorValues = false;
  vsmin = 8; vsmax = 16; vout = 6; iout = fromMili( 360 ); tambient = 85; rthja = 65; v_overhead = 5;
  rdson = 1; vgstyp = 2; r1_selected = 20.00; r2_selected = 8.10;
  vin_set = 1.22; k1 = 900; k2 = 890; ipsmax = fromMili( 500 ); vgs_pwr_sh = 5.50; vsmax_spec = 5.50;
  vsmin_spec = 40; iout_min = fromMili( 13.5 ); iout_max = fromMili( 360 ); vps_cc = 0.75; vout_cc = 1.40; tjmax = 150.00;

  // Check if output variables have correct values
  assert.ok( isNumberInRange(toKilo(rset), 3.01, 3.03), "rset ≈ 3.02 kΩ" );
  assert.ok( isNumberInRange(pvmax, 0.99, 1.01), "pvmax ≈ 1.00 W" );
  assert.ok( isNumberInRange(rshunt, 0.74, 0.76), "rshunt ≈ 0.75 Ω" ); // Input value which changes
  assert.ok( isNumberInRange(r1, 21.24, 21.26), "r1 ≈ 21.25 Ω" );
  assert.ok( isNumberInRange(r2, 6.99, 7.01), "r2 ≈ 7.00 Ω" );
  assert.ok( isNumberInRange(p1114max, 0.98, 1.00), "p1114max ≈ 0.99 W" );
  assert.ok( isNumberInRange(pr1_max, 2.35, 2.37), "pr1_max ≈ 2.36 W" );
  assert.ok( isNumberInRange(pr2_max, 0.95, 0.97), "pr2_max ≈ 0.96 W" );
  
  assert.ok( cfg_setting == 'GND', "cfg_setting = GND" );
  assert.ok( output_cfg == 'OUTH', "output_cfg = OUTH" );
  assert.ok( result == 'pass', "result = pass" );
  assert.ok( result_description == 'OK', "result_description = OK" );
});

// ------------------------------------------------------------------------------ #2

QUnit.test( "Formula correctness test - Test Case #2", function( assert ) {
  // Set input values and product parameters
  toolMOSFETVersion = 1; useRecommendedResistorValues = false;
  vsmin = 8; vsmax = 16; vout = 2; iout = fromMili( 360 ); tambient = 85; rthja = 65; v_overhead = 5;
  rdson = 1; vgstyp = 2; r1_selected = 20.00; r2_selected = 8.10;
  vin_set = 1.22; k1 = 900; k2 = 890; ipsmax = fromMili( 500 ); vgs_pwr_sh = 5.50; vsmax_spec = 5.50;
  vsmin_spec = 40; iout_min = fromMili( 13.5 ); iout_max = fromMili( 360 ); vps_cc = 0.75; vout_cc = 1.40; tjmax = 150.00;

  // Start the calculation
  calculateData();

  // Set input values and product parameters
  toolMOSFETVersion = 1; useRecommendedResistorValues = false;
  vsmin = 8; vsmax = 16; vout = 2; iout = fromMili( 360 ); tambient = 85; rthja = 65; v_overhead = 5;
  rdson = 1; vgstyp = 2; r1_selected = 20.00; r2_selected = 8.10;
  vin_set = 1.22; k1 = 900; k2 = 890; ipsmax = fromMili( 500 ); vgs_pwr_sh = 5.50; vsmax_spec = 5.50;
  vsmin_spec = 40; iout_min = fromMili( 13.5 ); iout_max = fromMili( 360 ); vps_cc = 0.75; vout_cc = 1.40; tjmax = 150.00;

  // Check if output variables have correct values
  assert.ok( isNumberInRange(toKilo(rset), 3.01, 3.03), "rset ≈ 3.02 kΩ" );
  assert.ok( isNumberInRange(pvmax, 0.99, 1.01), "pvmax ≈ 1.00 W" );
  assert.ok( isNumberInRange(rshunt, 0.74, 0.76), "rshunt ≈ 0.75 Ω" ); // Input value which changes
  assert.ok( isNumberInRange(r1, 0.00, 0.00), "r1 ≈ 0.00 Ω" );
  assert.ok( isNumberInRange(r2, 0.00, 0.00), "r2 ≈ 0.00 Ω" );
  assert.ok( isNumberInRange(p1114max, 0.97, 0.99), "p1114max ≈ 0.98 W" );
  assert.ok( isNumberInRange(pr1_max, 3.72, 3.74), "pr1_max ≈ 3.73 W" );
  assert.ok( isNumberInRange(pr2_max, 1.50, 1.52), "pr2_max ≈ 1.51 W" );
  
  assert.ok( cfg_setting == 'GND', "cfg_setting = GND" );
  assert.ok( output_cfg == 'OUTH', "output_cfg = OUTH" );
  assert.ok( result == 'pass', "result = pass" );
  assert.ok( result_description == 'OK', "result_description = OK" );
});

// ------------------------------------------------------------------------------ #3

QUnit.test( "Formula correctness test - Test Case #3", function( assert ) {
  // Set input values and product parameters
  toolMOSFETVersion = 1; useRecommendedResistorValues = false;
  vsmin = 8; vsmax = 16; vout = 6; iout = fromMili( 100 ); tambient = 85; rthja = 65; v_overhead = 5;
  rdson = 1; vgstyp = 2; r1_selected = 20.00; r2_selected = 8.10;
  vin_set = 1.22; k1 = 900; k2 = 890; ipsmax = fromMili( 500 ); vgs_pwr_sh = 5.50; vsmax_spec = 5.50;
  vsmin_spec = 40; iout_min = fromMili( 13.5 ); iout_max = fromMili( 360 ); vps_cc = 0.75; vout_cc = 1.40; tjmax = 150.00;

  // Start the calculation
  calculateData();

  // Set input values and product parameters
  toolMOSFETVersion = 1; useRecommendedResistorValues = false;
  vsmin = 8; vsmax = 16; vout = 6; iout = fromMili( 100 ); tambient = 85; rthja = 65; v_overhead = 5;
  rdson = 1; vgstyp = 2; r1_selected = 20.00; r2_selected = 8.10;
  vin_set = 1.22; k1 = 900; k2 = 890; ipsmax = fromMili( 500 ); vgs_pwr_sh = 5.50; vsmax_spec = 5.50;
  vsmin_spec = 40; iout_min = fromMili( 13.5 ); iout_max = fromMili( 360 ); vps_cc = 0.75; vout_cc = 1.40; tjmax = 150.00;

  // Check if output variables have correct values
  assert.ok( isNumberInRange(toKilo(rset), 10.97, 10.99), "rset ≈ 10.98 kΩ" );
  assert.ok( isNumberInRange(pvmax, 0.99, 1.01), "pvmax ≈ 1.00 W" );
  assert.ok( isNumberInRange(rshunt, 1.49, 1.51), "rshunt ≈ 1.50 Ω" ); // Input value which changes
  assert.ok( isNumberInRange(r1, 1073.00, 1075.00), "r1 ≈ 1074.00 Ω" );
  assert.ok( isNumberInRange(r2, 349.00, 351.00), "r2 ≈ 350.00 Ω" );
  assert.ok( isNumberInRange(p1114max, 0.19, 0.21), "p1114max ≈ 0.20 W" );
  assert.ok( isNumberInRange(pr1_max, 2.35, 2.37), "pr1_max ≈ 2.36 W" );
  assert.ok( isNumberInRange(pr2_max, 0.95, 0.97), "pr2_max ≈ 0.96 W" );
  
  assert.ok( cfg_setting == 'open', "cfg_setting = open" );
  assert.ok( output_cfg == 'OUTL', "output_cfg = OUTL" );
  assert.ok( result == 'pass', "result = pass" );
  assert.ok( result_description == 'OK', "result_description = OK" );
});

// ------------------------------------------------------------------------------ #4

QUnit.test( "Formula correctness test - Test Case #4", function( assert ) {
  // Set input values and product parameters
  toolMOSFETVersion = 1; useRecommendedResistorValues = false;
  vsmin = 8; vsmax = 16; vout = 6; iout = fromMili( 200 ); tambient = 85; rthja = 65; v_overhead = 5;
  rdson = 1; vgstyp = 2; r1_selected = 20.00; r2_selected = 8.10;
  vin_set = 1.22; k1 = 900; k2 = 890; ipsmax = fromMili( 500 ); vgs_pwr_sh = 5.50; vsmax_spec = 5.50;
  vsmin_spec = 40; iout_min = fromMili( 13.5 ); iout_max = fromMili( 360 ); vps_cc = 0.75; vout_cc = 1.40; tjmax = 150.00;

  // Start the calculation
  calculateData();

  // Set input values and product parameters
  toolMOSFETVersion = 1; useRecommendedResistorValues = false;
  vsmin = 8; vsmax = 16; vout = 6; iout = fromMili( 200 ); tambient = 85; rthja = 65; v_overhead = 5;
  rdson = 1; vgstyp = 2; r1_selected = 20.00; r2_selected = 8.10;
  vin_set = 1.22; k1 = 900; k2 = 890; ipsmax = fromMili( 500 ); vgs_pwr_sh = 5.50; vsmax_spec = 5.50;
  vsmin_spec = 40; iout_min = fromMili( 13.5 ); iout_max = fromMili( 360 ); vps_cc = 0.75; vout_cc = 1.40; tjmax = 150.00;

  // Check if output variables have correct values
  assert.ok( isNumberInRange(toKilo(rset), 5.42, 5.44), "rset ≈ 5.43 kΩ" );
  assert.ok( isNumberInRange(pvmax, 0.99, 1.01), "pvmax ≈ 1.00 W" );
  assert.ok( isNumberInRange(rshunt, 0.74, 0.76), "rshunt ≈ 0.75 Ω" ); // Input value which changes
  assert.ok( isNumberInRange(r1, 73.16, 73.18), "r1 ≈ 73.17 Ω" );
  assert.ok( isNumberInRange(r2, 23.32, 23.34), "r2 ≈ 23.33 Ω" );
  assert.ok( isNumberInRange(p1114max, 0.39, 0.41), "p1114max ≈ 0.40 W" );
  assert.ok( isNumberInRange(pr1_max, 2.35, 2.37), "pr1_max ≈ 2.36 W" );
  assert.ok( isNumberInRange(pr2_max, 0.95, 0.97), "pr2_max ≈ 0.96 W" );
  
  assert.ok( cfg_setting == 'GND', "cfg_setting = GND" );
  assert.ok( output_cfg == 'OUTH', "output_cfg = OUTH" );
  assert.ok( result == 'pass', "result = pass" );
  assert.ok( result_description == 'OK', "result_description = OK" );
});

/*
// ------------------------------------------------------------------------------
// TODO if needed
QUnit.test( "Formula correctness test - generatePowerData()", function( assert ) {

});

// ------------------------------------------------------------------------------
// TODO if needed
QUnit.test( "Formula correctness test - calculateResistors()", function( assert ) {

});

// ------------------------------------------------------------------------------
*/
