var test_environment = true;

// The function create2DArray(rows, cols) creates an empty 2D array with specified number of rows and columns.
QUnit.test( "test function create2DArray(rows, cols)", function( assert ) {

  let array2d;
  assert.ok( array2d === undefined, "Array is null (before assignment)" );

  array2d = create2DArray(5, 3);

  assert.ok( array2d !== undefined, "Array not null (after assignment)" );
  assert.ok( array2d.length == 5, "Array has correct number of rows" );
  assert.ok( array2d[0].length == 3, "Array has correct number of columns" );

  let allNulls = true, equalNumCols = true;

  for (var i = 0; i < array2d.length; i++) {

    if (array2d[i].length !== 3) {
      equalNumCols = false;
    }

    for (var j = 0; j < array2d[i].length; j++) {
      if (array2d[i][j] !== null) {
        allNulls = false;
      }
    }
  }

  assert.ok( equalNumCols, "Array is rectangular" );

});

// The function extractColumnFrom2DArray(array2d, column_id) returns one column from the 2D array as a 1D array.
QUnit.test( "test function extractColumnFrom2DArray(array2d, column_id)", function( assert ) {

  let array2d = create2DArray(3, 3);

  array2d[0][0] = 1; array2d[1][0] = 4; array2d[2][0] = 7;
  array2d[0][1] = 2; array2d[1][1] = 5; array2d[2][1] = 8;
  array2d[0][2] = 3; array2d[1][2] = 6; array2d[2][2] = 9;

  let extract1 = extractColumnFrom2DArray(array2d, 0);
  let extract2 = extractColumnFrom2DArray(array2d, 1);
  let extract3 = extractColumnFrom2DArray(array2d, 2);

  assert.ok(extract1[0] == 1 && extract1[1] == 4 && extract1[2] == 7, "Column 0");
  assert.ok(extract2[0] == 2 && extract2[1] == 5 && extract2[2] == 8, "Column 1");
  assert.ok(extract3[0] == 3 && extract3[1] == 6 && extract3[2] == 9, "Column 2");

  assert.ok(extract1.length == 3 && extract2.length == 3 && extract3.length == 3, "Length test");

  let notExists1 = extractColumnFrom2DArray(array2d, -1);
  let notExists2 = extractColumnFrom2DArray(array2d, +3);

  assert.ok(notExists1 === undefined, "Negative column ID");
  assert.ok(notExists2 === undefined, "Non-existent column ID");
});

// The function multiplyArrayByScalar(numberArray, scalar) multiplies all numbers from the given 1D array with a scalar and returns result as a 1D array.
QUnit.test( "test function multiplyArrayByScalar(numberArray, scalar)", function( assert ) {

  let array1d = [0, 1, 2, 3, -1, -2, -3];

  let arrayMultiplied1 = multiplyArrayByScalar(array1d, -2);
  let arrayMultiplied2 = multiplyArrayByScalar(array1d, +2);

  assert.ok(JSON.stringify(arrayMultiplied1) === JSON.stringify([0, -2, -4, -6, 2, 4, 6]), "Test negative scalar");
  assert.ok(JSON.stringify(arrayMultiplied2) === JSON.stringify([0, 2, 4, 6, -2, -4, -6]), "Test positive scalar");

});

// The function toFixedNumberArray(numberArray, decimalPoints) rounds every number in a 1D array to specified number of decimal points.
QUnit.test( "test function toFixedNumberArray(numberArray, decimalPoints)", function( assert ) {

  let array1d = [0.12345, 1.23456, 2.34567, 3.45678, 4.56789, 5.67890, 6.78901, 7.89012, 8.90123, 9.01234];

  let expected0 = [0, 1, 2, 3, 5, 6, 7, 8, 9, 9];
  let expected1 = [0.1, 1.2, 2.3, 3.5, 4.6, 5.7, 6.8, 7.9, 8.9, 9.0];
  let expected2 = [0.12, 1.23, 2.35, 3.46, 4.57, 5.68, 6.79, 7.89, 8.90, 9.01];
  let expected3 = [0.123, 1.235, 2.346, 3.457, 4.568, 5.679, 6.789, 7.890, 8.901, 9.012];
  let expected4 = [0.1235, 1.2346, 2.3457, 3.4568, 4.5679, 5.6789, 6.7890, 7.8901, 8.9012, 9.0123];
  let expected5 = [0.12345, 1.23456, 2.34567, 3.45678, 4.56789, 5.67890, 6.78901, 7.89012, 8.90123, 9.01234];
  let expected6 = [0.12345, 1.23456, 2.34567, 3.45678, 4.56789, 5.67890, 6.78901, 7.89012, 8.90123, 9.01234];

  let rounded0 = toFixedNumberArray(array1d, 0);
  let rounded1 = toFixedNumberArray(array1d, 1);
  let rounded2 = toFixedNumberArray(array1d, 2);
  let rounded3 = toFixedNumberArray(array1d, 3);
  let rounded4 = toFixedNumberArray(array1d, 4);
  let rounded5 = toFixedNumberArray(array1d, 5);
  let rounded6 = toFixedNumberArray(array1d, 6);

  assert.ok(compare1DArrays(rounded0, expected0), "expected0");
  assert.ok(compare1DArrays(rounded1, expected1), "expected1");
  assert.ok(compare1DArrays(rounded2, expected2), "expected2");
  assert.ok(compare1DArrays(rounded3, expected3), "expected3");
  assert.ok(compare1DArrays(rounded4, expected4), "expected4");
  assert.ok(compare1DArrays(rounded5, expected5), "expected5");
  assert.ok(compare1DArrays(rounded6, expected6), "expected6");
});

// The function roundDecimals(number, ndecimals) rounds the number to ndecimals decimal places.
QUnit.test( "test function roundDecimals(number, ndecimals)", function( assert ) {

  let num1 = 10.123456789;

  let round0 = roundDecimals(num1, 0);
  let round1 = roundDecimals(num1, 1);
  let round2 = roundDecimals(num1, 2);
  let round3 = roundDecimals(num1, 3);
  let round4 = roundDecimals(num1, 4);
  let round5 = roundDecimals(num1, 5);
  let round6 = roundDecimals(num1, 6);
  let round7 = roundDecimals(num1, 7);
  let round8 = roundDecimals(num1, 8);
  let round9 = roundDecimals(num1, 9);
  let round10 = roundDecimals(num1, 10);

  assert.ok(round0, "round0");
  assert.ok(round1, "round1");
  assert.ok(round2, "round2");
  assert.ok(round3, "round3");
  assert.ok(round4, "round4");
  assert.ok(round5, "round5");
  assert.ok(round6, "round6");
  assert.ok(round7, "round7");
  assert.ok(round8, "round8");
  assert.ok(round9, "round9");
  assert.ok(round10, "round10");

});

// The function findColumnMaxValueIn2DArray(array2d, column_index) finds maximum value from a column of a 2D array and returns it as a single scalar value.
QUnit.test( "test function findColumnMaxValueIn2DArray(array2d, column_index)", function( assert ) {
  
  let array2d = create2DArray(3, 3);

  array2d[0][0] = 1; array2d[1][0] = 4; array2d[2][0] = 7;
  array2d[0][1] = 5; array2d[1][1] = 8; array2d[2][1] = 2;
  array2d[0][2] = 9; array2d[1][2] = 3; array2d[2][2] = 6;

  let max0 = findColumnMaxValueIn2DArray(array2d, 0);
  let max1 = findColumnMaxValueIn2DArray(array2d, 1);
  let max2 = findColumnMaxValueIn2DArray(array2d, 2);

  assert.equal(max0, 7, "max0");
  assert.equal(max1, 8, "max1");
  assert.equal(max2, 9, "max2");

});

// Test all the unit prefix conversion functions.
QUnit.test( "test unit prefix conversion functions", function( assert ) {

  assert.ok( fromMili(5000) == 5, "fromMili(5000) == 5" );
  assert.ok( toMili(5) == 5000, "toMili(5) == 5000" );
  assert.ok( toKilo(5000) == 5, "fromKilo(5000) == 5" );
  assert.ok( fromKilo(5) == 5000, "toKilo(5) == 5000" );

});

// The function guardDependentParameters(lower_param_id, higher_param_id, last_updated_param_id) makes sure that the last updated parameter is inside the range defined by other two parameters.
QUnit.test( "test function guardDependentParameters(lower_param_id, higher_param_id, last_updated_param_id)", function( assert ) {

  // Create new empty param_map
  param_map = new Map();

  param_map.set('lower_param', {value: 5.00, min: 0.00, max: 'NaN'});
  param_map.set('higher_param', {value: 9.00, min: 'NaN', max: 12.00});

  guardDependentParameters('lower_param', 'higher_param', 'higher_param');
  guardDependentParameters('lower_param', 'higher_param', 'lower_param');

  assert.equal( param_map.get('lower_param').max, 9.00, 'Test lower' );
  assert.equal( param_map.get('higher_param').min, 5.00, 'Test higher' );

  assert.equal( param_map.get('lower_param').min, 0.00, 'Test lower unchanged' );
  assert.equal( param_map.get('higher_param').max, 12.00, 'Test higher unchanged' );

  assert.equal( param_map.get('lower_param').value, 5.00, 'Test lower value unchanged' );
  assert.equal( param_map.get('higher_param').value, 9.00, 'Test higher value unchanged' );

});

// The function guardDependentParametersLowerFor(lower_param_id, higher_param_id, for_param_id, last_updated_param_id) makes sure lower_param_id.max <= higher_param_id.value - for_param_id.value.
QUnit.test( "test function guardDependentParametersLowerFor(lower_param_id, higher_param_id, for_param_id, last_updated_param_id)", function( assert ) {

  // Create new empty param_map
  param_map = new Map();

  param_map.set('lower_param', {value: 5.00, min: 0.00, max: 'NaN'});
  param_map.set('higher_param', {value: 9.00, min: 'NaN', max: 12.00});
  param_map.set('for_param', {value: 2.00, min: 'NaN', max: "NaN"});

  guardDependentParametersLowerFor('lower_param', 'higher_param', 'for_param', 'higher_param');
  assert.equal( param_map.get('lower_param').max, 7.00, 'Test 1' );

  param_map.set('for_param', {value: 3.00, min: 'NaN', max: "NaN"});
  guardDependentParametersLowerFor('lower_param', 'higher_param', 'for_param', 'higher_param');
  assert.equal( param_map.get('lower_param').max, 6.00, 'Test 2' );

  param_map.set('higher_param', {value: 6.00, min: 'NaN', max: 12.00});
  guardDependentParametersLowerFor('lower_param', 'higher_param', 'for_param', 'higher_param');
  assert.equal( param_map.get('lower_param').max, 3.00, 'Test 3' );

});

// The function isNumberInRange(number, minimum, maximum) should return true if given number is within the range [minimum, maximum].
QUnit.test( "test isNumberInRange(number, minimum, maximum)", function( assert ) {

  assert.ok( isNumberInRange(0.00, 0.00, 1.00), "Lower boundary" );
  assert.ok( isNumberInRange(1.00, 0.00, 1.00), "Higher boundary" );
  assert.ok( isNumberInRange(0.50, 0.00, 1.00), "Inside boundaries" );

  assert.notOk( isNumberInRange(-0.000000001, 0.00, 1.00), "Below lower boundary" );
  assert.notOk( isNumberInRange(+1.000000001, 0.00, 1.00), "Above higher boundary" );
  
});

// The function getParamValue(param_id) returns a value from the param_map variable with param_id.
QUnit.test( "test function getParamValue(param_id)", function( assert ) {

  // Create new empty param_map
  param_map = new Map();

  assert.throws(
    function() { getParamValue('invalid_id') },
    function(err) { return err.toString() === "TypeError: Cannot read property 'value' of undefined" },
    "Test exception (undefined)"
  );

  // Create a new variable inside the param_map
  param_map.set('valid_id', {});

  assert.ok( Number.isNaN(getParamValue('valid_id')), "Test undefined value" );

  // Set value of the new variable
  param_map.set('valid_id', {value: 5.01});

  assert.equal( getParamValue('valid_id'), 5.01, "Test equals");

});

/* Returns either true or false. */
function compare1DArrays(array1, array2) {
  let returns = array1.length === array2.length;

  if (!returns) return returns;

  for (let i = 0; i < array1.length; i++) {
    if (array1[i] != array2[i]) {
      returns = false;
    }
  }

  return returns;
}