// Global variable which if set to true will prevent running GUI related code
var test_environment = true;

// Clone original parameters map
param_map = cloneParamMap(PARAM_MAP);

QUnit.test( "Formula integrity check test - MOSFET version", function( assert ) {
  // Set tool version with MOSFET
  toolMOSFETVersion = 1;

  // Use calculated resitors
  useRecommendedResistorValues = true;

  // Set input values
  vsmin = getParamValue('vsmin');
  vsmax = getParamValue('vsmax');
  vout = getParamValue('vout');
  iout = fromMili( getParamValue('iout') );
  tambient = getParamValue('tambient');
  rthja = getParamValue('rthja');
  v_overhead = getParamValue('v_overhead');
  rdson = getParamValue('rdson');
  vgstyp = getParamValue('vgstyp');
  r1_selected = getParamValue('r1_selected');
  r2_selected = getParamValue('r2_selected');

  // Set product parameters (TLD1114-EP):
  vin_set = getParamValue('vin_set');
  k1 = getParamValue('k1');
  k2 = getParamValue('k2');
  ipsmax = fromMili( getParamValue('ipsmax') );
  vgs_pwr_sh = getParamValue('vgs_pwr_sh');
  rshunt = getParamValue('rshunt');
  vsmax_spec = getParamValue('vsmax_spec');
  vsmin_spec = getParamValue('vsmin_spec');
  iout_min = fromMili( getParamValue('iout_min') );
  iout_max = fromMili( getParamValue('iout_max') );
  vps_cc = getParamValue('vps_cc');
  vout_cc = getParamValue('vout_cc');
  tjmax = getParamValue('tjmax');


  // Start the calculation
  calculateData();

  // Check if output variables have correct values
  assert.ok( rset == (iout > 0.18 ? vin_set / iout * k2 : vin_set / iout * k1), "rset" );
  assert.ok( cfg_setting == (iout > 0.18 ? 'GND' : 'open'), "cfg_setting" );
  assert.ok( output_cfg == (iout > 0.18 ? 'OUTH' : 'OUTL'), "output_cfg" );
  assert.ok( pvmax == (tjmax - tambient) / rthja, "pvmax" );
  assert.ok( rshunt == (iout > 0.18 ? 0.75 : 0.75 * 2), "rshunt" );

  // Since the function calculateResistors() is huge it requires its 
  // own test case and therefore it will not be tested here.
  // Here will be tested only if the values have been correctly assigned.
  let resistors = calculateResistors();
  assert.ok( r1 == resistors[0], "r1" );
  assert.ok( r2 == resistors[1], "r2" );

  // Since the function generatePowerData() is huge it requires its 
  // own test case and therefore it will not be tested here.
  // Here will be tested only if the values have been correctly assigned.
  assert.ok( p1114max == findColumnMaxValueIn2DArray(powerData, 4), "p1114max" );
  assert.ok( pr1_max == findColumnMaxValueIn2DArray(powerData, 5), "pr1_max" );
  assert.ok( pr2_max == findColumnMaxValueIn2DArray(powerData, 6), "pr2_max" );

});

QUnit.test( "Formula integrity check test - non-MOSFET version", function( assert ) {
  // Set tool version with MOSFET
  toolMOSFETVersion = 2;

  // Use calculated resitors
  useRecommendedResistorValues = true;

  // Set input values
  vsmin = getParamValue('vsmin');
  vsmax = getParamValue('vsmax');
  vout = getParamValue('vout');
  iout = fromMili( getParamValue('iout') );
  tambient = getParamValue('tambient');
  rthja = getParamValue('rthja');
  v_overhead = getParamValue('v_overhead');
  rdson = getParamValue('rdson');
  vgstyp = getParamValue('vgstyp');
  r1_selected = getParamValue('r1_selected');
  r2_selected = getParamValue('r2_selected');

  // Set product parameters (TLD1114-EP):
  vin_set = getParamValue('vin_set');
  k1 = getParamValue('k1');
  k2 = getParamValue('k2');
  ipsmax = fromMili( getParamValue('ipsmax') );
  vgs_pwr_sh = getParamValue('vgs_pwr_sh');
  rshunt = getParamValue('rshunt');
  vsmax_spec = getParamValue('vsmax_spec');
  vsmin_spec = getParamValue('vsmin_spec');
  iout_min = fromMili( getParamValue('iout_min') );
  iout_max = fromMili( getParamValue('iout_max') );
  vps_cc = getParamValue('vps_cc');
  vout_cc = getParamValue('vout_cc');
  tjmax = getParamValue('tjmax');


  // Start the calculation
  calculateData();

  // Check if output variables have correct values
  assert.ok( rset == (iout > 0.18 ? vin_set / iout * k2 : vin_set / iout * k1), "rset" );
  assert.ok( cfg_setting == (iout > 0.18 ? 'GND' : 'open'), "cfg_setting" );
  assert.ok( output_cfg == (iout > 0.18 ? 'OUTH' : 'OUTL'), "output_cfg" );
  assert.ok( pvmax == (tjmax - tambient) / rthja, "pvmax" );
  assert.ok( rshunt == (iout > 0.18 ? 0.75 : 0.75 * 2), "rshunt" );

  // Since the function calculateResistors() is huge it requires its 
  // own test case and therefore it will not be tested here.
  // Here will be tested only if the values have been correctly assigned.
  let resistors = calculateResistors();
  assert.ok( r1 == resistors[0] + resistors[1], "r1" );
  assert.ok( r2 == 0.000000001, "r2" );

  // Since the function generatePowerData() is huge it requires its 
  // own test case and therefore it will not be tested here.
  // Here will be tested only if the values have been correctly assigned.
  assert.ok( p1114max == findColumnMaxValueIn2DArray(powerData, 4), "p1114max" );
  assert.ok( pr1_max == findColumnMaxValueIn2DArray(powerData, 5), "pr1_max" );
  assert.ok( pr2_max == findColumnMaxValueIn2DArray(powerData, 6), "pr2_max" );

  // Test overall results and result descriptions
  assert.ok( (p1114max / pvmax <= 1.00) == (result == 'pass' && result_description == 'OK'), "result pass" );
  assert.ok( (p1114max / pvmax > 1.00 && p1114max / pvmax <= 1.20) == (result == 'possible fail' && result_description == 'power high'), "result possible fail" );
  assert.ok( (p1114max / pvmax > 1.20) == (result == 'fail' && result_description == 'power high'), "result power high" );
});